//# DSWorldMarker.h : Implementation of a world coords DSMarker
//# Copyright (C) 1998,1999,2000,2001,2002
//# Associated Universities, Inc. Washington DC, USA.
//#
//# This library is free software; you can redistribute it and/or modify it
//# under the terms of the GNU Library General Public License as published by
//# the Free Software Foundation; either version 2 of the License, or (at your
//# option) any later version.
//#
//# This library is distributed in the hope that it will be useful, but WITHOUT
//# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
//# License for more details.
//#
//# You should have received a copy of the GNU Library General Public License
//# along with this library; if not, write to the Free Software Foundation,
//# Inc., 675 Massachusetts Ave, Cambridge, MA 02139, USA.
//#
//# Correspondence concerning AIPS++ should be addressed as follows:
//#        Internet email: casa-feedback@nrao.edu.
//#        Postal address: AIPS++ Project Office
//#                        National Radio Astronomy Observatory
//#                        520 Edgemont Road
//#                        Charlottesville, VA 22903-2475 USA
//#
//# $Id:
#ifndef TRIALDISPLAY_DSWORLDMARKER_H
#define TRIALDISPLAY_DSWORLDMARKER_H

#include <casa/aips.h>
#include <casa/Arrays/Vector.h>
#include <casa/Arrays/Matrix.h>

#include <casa/Quanta/Quantum.h>
#include <casa/Quanta/QuantumHolder.h>
#include <display/Display/PanelDisplay.h>

#include <display/DisplayShapes/DSMarker.h>
#include <display/DisplayShapes/DisplayShapeWithCoords.h>

namespace casa { //# NAMESPACE CASA - BEGIN

// <summary>
// Implementation of a world coords DSMarker
// </summary>

	class PanelDisplay;
	class DSPixelMarker;
	class DSScreenMarker;

	class DSWorldMarker : public DSMarker , public DisplayShapeWithCoords {

	public:

		DSWorldMarker();
		DSWorldMarker(const casacore::Record& settings, PanelDisplay* pd);
		DSWorldMarker(DSScreenMarker& other, PanelDisplay* pd);
		DSWorldMarker(DSPixelMarker& other, PanelDisplay* pd);

		virtual ~DSWorldMarker();

		virtual void recalculateScreenPosition();

		virtual void move(const casacore::Float& dX, const casacore::Float& dY);
		virtual void setCenter(const casacore::Float& xPos, const casacore::Float& yPos);
		virtual void draw(PixelCanvas* pc);

		virtual casacore::Bool setOptions(const casacore::Record& settings);
		virtual casacore::Record getOptions();

		virtual casacore::Record getRawOptions() {
			return DSMarker::getOptions();
		}

		virtual PanelDisplay* panelDisplay()  {
			return itsPD;
		}
	private:

		// Based on a WC option record
		//  virtual casacore::Bool chooseWC(const casacore::Record& settings, PanelDisplay* pd,
		//		WorldCanvas* wc);

		// Based on a pixel center
		//  virtual casacore::Bool chooseWC(const casacore::Float& xPos, const casacore::Float& yPos,
		//		PanelDisplay* pd,
		//		WorldCanvas* wc);

		// The paneldisplay from which I may choose an appropriate WC
		PanelDisplay* itsPD;

		// The WC of my choosing
		WorldCanvas* itsWC;

		// The center of the marker in world co-ords.
		casacore::Vector<casacore::Quantum<casacore::Double> > itsWorldCenter;

		void updateWCent();
	};

} //# NAMESPACE CASA - END

#endif



